/*
 * rlimit.c
 *
 * getrlimit, setrlimit bindings
 *
 * Author  : Hye-Shik Chang <perky@fallin.lv>
 * Date    : $Date: 2002/06/13 17:02:12 $
 * Created : 13 June 2002
 *
 * $Revision: 1.2 $
 */


/*
 * Constants Listing
 */
#if defined(PYFB_CONSTANTS)

SET_INTCONSTANT(RLIMIT_CORE)
SET_INTCONSTANT(RLIMIT_CPU)
SET_INTCONSTANT(RLIMIT_DATA)
SET_INTCONSTANT(RLIMIT_FSIZE)
SET_INTCONSTANT(RLIMIT_MEMLOCK)
SET_INTCONSTANT(RLIMIT_NOFILE)
SET_INTCONSTANT(RLIMIT_NPROC)
SET_INTCONSTANT(RLIMIT_RSS)
SET_INTCONSTANT(RLIMIT_STACK)
SET_INTCONSTANT(RLIMIT_SBSIZE)


/*
 * Methods Listing
 */
#elif defined(PYFB_METHODS)

METHOD(getrlimit),
METHOD(setrlimit),


/*
 * Function Bindings
 */
#else


static char Py_getrlimit__doc__[] =
"getrlimit(resource): get maximum system resource consumption";

static PyObject *
Py_getrlimit(PyObject *self, PyObject *args)
{
    struct rlimit rlp;
    PyObject *r;
    int resource;

    if (! PyArg_ParseTuple(args, "i:getrlimit", &resource))
        return NULL;

    if (getrlimit(resource, &rlp) == -1) {
        PyErr_SetFromErrno(PyExc_OSError);
        return NULL;
    }

    r = PyTuple_New(2);

    if (rlp.rlim_cur == RLIM_INFINITY) {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(r, 0, Py_None);
    } else
        PyTuple_SET_ITEM(r, 0, PyLong_FromLongLong(rlp.rlim_cur));

    if (rlp.rlim_max == RLIM_INFINITY) {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(r, 1, Py_None);
    } else
        PyTuple_SET_ITEM(r, 1, PyLong_FromLongLong(rlp.rlim_max));

    return r;
}


static char Py_setrlimit__doc__[] =
"setrlimit(resource, rlim_cur, rlim_max): set maximum system resource consumption";

static PyObject *
Py_setrlimit(PyObject *self, PyObject *args)
{
    struct rlimit rlp;
    int resource;

    if (! PyArg_ParseTuple(args, "iLL:setrlimit", &resource, &(rlp.rlim_cur), &(rlp.rlim_max)))
        return NULL;

    if (setrlimit(resource, &rlp) == -1) {
        PyErr_SetFromErrno(PyExc_OSError);
        return NULL;
    }

    Py_INCREF(Py_None);
    return Py_None;
}

#endif
